Collection subclass: #BinaryTree
	instanceVariableNames: 'leftNode rightNode value tally '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex15.4'!
BinaryTree comment:
'This class is the answer to exercise 15.2. It represents the node in a binary tree.

Instance Variables:
leftNode		<BinaryTree | nil> the root of my left tree
rightNode	<BinaryTree | nil> the root of my right tree
value		<Object> the element at this node. (Must repsond to #> and #<)
tally			<Integer> number of occurrences of value'!


!BinaryTree methodsFor: 'removing'!

remove: oldObject ifAbsent: anExceptionBlock 
	"Remove oldObject as one of the receiver's elements. If no element is equal to 
	oldObject, answer the result of evaluating anExceptionBlock. Otherwise, 
	answer the argument, oldObject."

	value = oldObject ifTrue: [^tally <= 0
			ifFalse: 
				[tally := tally - 1.
				oldObject]
			ifTrue: [anExceptionBlock value]].
	oldObject < value ifTrue: [^leftNode isNil
			ifTrue: [anExceptionBlock value]
			ifFalse: [leftNode remove: oldObject ifAbsent: anExceptionBlock]].
	oldObject > value ifTrue: [^rightNode isNil
			ifTrue: [anExceptionBlock value]
			ifFalse: [rightNode remove: oldObject ifAbsent: anExceptionBlock]]! !

!BinaryTree methodsFor: 'enumerating'!

do: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. (in ascending order). (Alternatively, we could implement this as 'self elements do: aBlock')"

	leftNode notNil ifTrue:[leftNode do: aBlock].
	tally timesRepeat:[aBlock value: value].
	rightNode notNil ifTrue:[rightNode do: aBlock]! !

!BinaryTree methodsFor: 'adding'!

add: aValue 
	"Only add aValue if it is the same class as my value"

	aValue class = value class ifFalse: [^self].
	aValue = value ifTrue:[tally := tally + 1].
	aValue < value ifTrue: [self addLeft: aValue].
	aValue > value ifTrue: [self addRight: aValue].
	^aValue! !

!BinaryTree methodsFor: 'accessing'!

elements
	^self asOrderedCollection! !

!BinaryTree methodsFor: 'private'!

addLeft: aValue 
	leftNode isNil
		ifTrue: [leftNode := self class value: aValue]
		ifFalse: [leftNode add: aValue]!

addRight: aValue 
	rightNode isNil
		ifTrue: [rightNode := self class value: aValue]
		ifFalse: [rightNode add: aValue]!

asOrderedCollection
	| collection |
	collection := OrderedCollection new.
	self insertInto: collection.
	^collection!

insertInto: aCollection
	leftNode notNil ifTrue:[leftNode insertInto: aCollection].
	tally timesRepeat: [aCollection add: value].
	rightNode notNil ifTrue:[rightNode insertInto: aCollection]!

setValue: aValue
	value := aValue.
	tally := 1.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BinaryTree class
	instanceVariableNames: ''!


!BinaryTree class methodsFor: 'instance creation'!

value: aValue
	^self new setValue: aValue! !

!BinaryTree class methodsFor: 'examples'!

example
	"Be able to add to and return the elements from a BinaryTree"
	"BinaryTree example"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	^tree elements 	"return objects in ascending order"!

exampleDo
	"Be able to enumerate over the elements of a BinaryTree"
	"BinaryTree exampleDo"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	^tree do:[:i | Transcript show: i printString; cr] "Enumerate the tree"!

examplePrintString
	"Be able to print the elements from a BinaryTree"
	"BinaryTree examplePrintString"

	| tree |
	tree := self value: 6.
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | "Create a new tree whose object is 6"
		tree add: i].	"add objects to the tree"
	"print the objects in ascending order"
	^tree printString!

exampleRemoveAbsent
	"Be able to remove an element that is absent"
	"BinaryTree exampleRemoveAbsent"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	tree remove: 5 ifAbsent: [Screen default ringBell].
	tree remove: 6; remove: 6; remove: 6. "Throws an error"!

exampleRemoveAll
	"Be able to remove a collection of elements"
	"BinaryTree exampleRemoveAll"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	^tree removeAll: #(6  3 )!

exampleRemovePresent
	"Be able to remove an element that is present"
	"BinaryTree exampleRemovePresent"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	tree remove: 67; remove: 6.
	^tree!

exampleSize
	"Be able to enumerate over the elements of a BinaryTree"
	"BinaryTree exampleSize"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	^tree size "Count the elements of tree"!

exampleSortedCollection
	"Be able to return the elements from a BinaryTree in a SortedCollection"
	"BinaryTree exampleSortedCollection"

	| tree |
	tree := self value: 6. "Create a new tree whose object is 6"
	#(12 34 #e 67 $e 89 'e' 10 3 6) do: [:i | 
		tree add: i].	"add objects to the tree"
	^tree asSortedCollection:[:x :y | x > y] 	"return objects in descending order"! !

