Semaphore subclass: #Chopstick
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex25.16'!
Chopstick comment:
'I represent a Chopstick available for use by a Philsopher. I''m really just a Semaphore with a more suitable ''dining'' interface. I also overrride #new so that instance of me are ready to be picked  up.'!


!Chopstick methodsFor: 'accessing'!

pickUp
	"Try to pick me up, wait if I'm currently being used"

	self wait!

putDown
	"Put me down and let anyone else pick me up if they want me"

	self signal! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Chopstick class
	instanceVariableNames: ''!


!Chopstick class methodsFor: 'instance creation'!

new
	"Create a new instance of me, with a signal outstanding"

	^super new signal! !

Object subclass: #DiningPhilosophers
	instanceVariableNames: 'philosophers '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex25.16'!
DiningPhilosophers comment:
'I represent an interface to the 5 philosophers. 
See my class example. Send me the following:

startDining		to get my philosophers to sit down to dinner.
suspendDining	to take a break
stopDining		when you''re thru. Remember to do this before closing the inspector.

Instance Variables:
philosophers		<Set>	my 5 philosophers'!


!DiningPhilosophers methodsFor: 'initialize-release'!

initialize
	philosophers := Set new: 5.
	self setup! !

!DiningPhilosophers methodsFor: 'private'!

setup
	"Create the philosophers and their chopsticks"

	| karl sigmund bernard trevor prentice c1 c2 c3 c4 c5 |
	karl := Philosopher named: 'Marx'.
	sigmund := Philosopher named: 'Freud'.
	bernard := Philosopher named: 'Horan'.
	trevor := Philosopher named: 'Hopkins'.
	prentice := Philosopher named: 'Hall'.
	philosophers add: karl; add: sigmund; add: bernard; add: trevor; add: prentice.
	c1 := Chopstick new.
	c2 := Chopstick new.
	c3 := Chopstick new.
	c4 := Chopstick new.
	c5 := Chopstick new.
	karl leftChopstick: c5 rightChopstick: c1.
	prentice leftChopstick: c1 rightChopstick: c2.
	trevor leftChopstick: c2 rightChopstick: c3.
	bernard leftChopstick: c3 rightChopstick: c4.
	sigmund leftChopstick: c4 rightChopstick: c5!

startDining
	"Sit down for dinner"

	philosophers do: [:p | p startDining]!

stopDining
	"Finish dinner. 
	Do this to stop all the processes"

	philosophers do: [:p | p suspendDining]!

suspendDining
	"Take a break from dinner"

	philosophers do: [:p | p suspendDining]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DiningPhilosophers class
	instanceVariableNames: ''!


!DiningPhilosophers class methodsFor: 'examples'!

example
	"Make sure you inspect the result. 
	Send me a message such as: 
	#suspendDining or #stopDining when you're finished"
	"DiningPhilosophers example"

	| aDP |
	aDP := self new.
	aDP startDining.
	^aDP! !

!DiningPhilosophers class methodsFor: 'instance creation'!

new
	^super new initialize! !

Object subclass: #Philosopher
	instanceVariableNames: 'name leftChopstick rightChopstick random diningProcess '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex25.16'!
Philosopher comment:
'I represent one of the 5 philosophers in exercise 25.16.  I eat & think, quite a good life.

Instance Variables:
name			<String>		my name. Used for tracing on the Transcript.
leftChopstick		<Chopstick>	the Chopstick to my left
rightChopstick	<Chopstick>	the Chopstick to my right
random			<Random>	gives some indeterminancy in my life
diningProcess	<Process>	the eating and thinking

See class DiningPhilosophers for an example'!


!Philosopher methodsFor: 'eating and thinking'!

acquireChopsticks
	"Get the chopsticks so I can eat"

	Transcript cr; show: name , ' is waiting'; tab; show: Time now printString.
	leftChopstick pickUp.
	rightChopstick pickUp!

eat
	"Get the chopsticks. 
	Take my time eating. 
	Put the chopsticks down"

	| delay |
	self acquireChopsticks.
	Transcript cr; show: name , ' is eating'; tab; show: Time now printString.
	delay := Delay forSeconds: (random next * 6 + 1) truncated.
	delay wait.
	self putDownChopsticks!

putDownChopsticks
	"I've finished eating, so I can put the chopsticks down"

	leftChopstick putDown.
	rightChopstick putDown!

startDining
	"Sit down for dinner. 
	I.E. start the process for my eating & thinking"

	diningProcess resume!

stopDining
	"Finish dinner. 
	I.E. terminate my eating & thinking process"

	diningProcess terminate!

suspendDining
	"Take a rest from dinner (maybe a bio-break?). 
	I.E. suspend my eating & thinking process"

	diningProcess suspend!

think
	"Oooh -- have wonderful thoughts"

	| delay |
	delay := Delay forSeconds: (random next * 6 + 1) truncated.
	delay wait! !

!Philosopher methodsFor: 'private'!

setName: aString 
	"Create a random number generator. 
	Create my eating & thinkiing process. 
	Give myself a name"

	random := Random new.
	diningProcess := [
			[self eat.
			self think] repeat] newProcess.
	name := aString! !

!Philosopher methodsFor: 'accessing'!

leftChopstick: lChopstick rightChopstick: rChopstick 
	"Identify the chopsticks I can use"

	rightChopstick := rChopstick.
	leftChopstick := lChopstick! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Philosopher class
	instanceVariableNames: ''!


!Philosopher class methodsFor: 'instance creation'!

named: aString
	^self new setName: aString! !

