Object subclass: #Clock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dependency-Examples'!


!Clock methodsFor: 'cycling'!

cycle
	"Perform one clock cycle"

	self changed: #risingEdge.
	self changed: #fallingEdge! !

Object subclass: #FlipFlop
	instanceVariableNames: 'input state newState '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dependency-Examples'!


!FlipFlop methodsFor: 'accessing'!

clear
	"Set the state to false, immediately"

	state := false.
	newState := false!

input: aLogicUnit 
	"Set the input to be aLogicUnit"

	input := aLogicUnit!

preset
	"Set the state to true, immediately"

	state := true.
	newState := true!

state
	"Answer with the state of the receiver"

	^state! !

!FlipFlop methodsFor: 'initialize-release'!

initialize
	"Initialize the receiver"

	self clear!

release
	"Nil out any references to remove possible cycles"

	input := nil.
	super release! !

!FlipFlop methodsFor: 'clocking'!

negativeClock
	"copy the input value into the receiver's new state"

	state := newState!

positiveClock
	"copy the input value into the receiver's new state"

	newState := input state! !

!FlipFlop methodsFor: 'updating'!

update: aParameter
	"Perform the clocking action on a rising edge"
	aParameter == #risingEdge ifTrue:[self positiveClock].
	aParameter == #fallingEdge ifTrue:[self negativeClock].! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FlipFlop class
	instanceVariableNames: ''!


!FlipFlop class methodsFor: 'instance creation'!

new
	"Create a new initialized instance of the receiver"

	^super new initialize! !

