ApplicationModel subclass: #VWDice
	instanceVariableNames: 'value diceModel '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex34.5'!
VWDice comment:
'This class represents an application model that emulates a dice (Ex 34.5).

Instance variables:
diceModel	<Dice>			provides a random number between 1 and 6
value		<ValueHolder>	containing the value of the diceModel'!


!VWDice methodsFor: 'modifying'!

rollDice
	"Roll the dice."

	diceModel roll.
	value value: diceModel value! !

!VWDice methodsFor: 'accessing'!

diceView
	"Return a diceView on the diceModel"

	^DiceView model: diceModel! !

!VWDice methodsFor: 'initialize-release'!

initialize
	"Initialize the instance variables of the receiver"

	diceModel := Dice new.
	value := ValueHolder with: diceModel value.! !

!VWDice methodsFor: 'aspects'!

value
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^value isNil
		ifTrue:
			[value := 0 asValue]
		ifFalse:
			[value]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VWDice class
	instanceVariableNames: ''!


!VWDice class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'VW Dice' 
			#min: #(#Point 228 243 ) 
			#max: #(#Point 228 243 ) 
			#bounds: #(#Rectangle 165 156 393 399 ) 
			#flags: 4 
			#menu: #diceMenu ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#Point 18 17 ) 
					#label: 'Value:' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 69 18 169 36 ) 
					#model: #value 
					#tabable: false 
					#isReadOnly: true 
					#type: #number 
					#formatString: '0' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 82 50 129 75 ) 
					#model: #rollDice 
					#tabable: false 
					#label: 'Roll' 
					#isDefault: true 
					#defaultable: true ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 20 87 208 192 ) 
					#component: #diceView ) ) ) )! !

!VWDice class methodsFor: 'resources'!

diceMenu
	"UIMenuEditor new openOnClass: self andSelector: #diceMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#label: 'Dice' 
				#submenu: #(#Menu #(
						#(#MenuItem 
							#label: 'Roll' ) ) #(1 ) #(#rollDice ) ) ) ) #(1 ) #(nil ) ) decodeAsLiteralArray! !

View subclass: #DiceView
	instanceVariableNames: 'spotWidth spotHeight spot '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex34.5'!
DiceView comment:
'I am the view that displays the dice.

Instance Variables:
spot			<EllipticalArc>	represents a dot to be displayed
spotHeight	<Integer>		the height of the spot
spotWidth	<Integer>		the width of the spot'!


!DiceView methodsFor: 'displaying'!

displayOn: aGraphicsContext 
	| box cent offset |
	box := self bounds.
	cent := box center.
	offset := cent - (spotWidth // 2 @ (spotHeight // 2)).
	(#(1 3 5) includes: model value)
		ifTrue: [spot displayOn: aGraphicsContext at: offset].
	(#(2 3 4 5 6) includes: model value)
		ifTrue: 
			[spot displayOn: aGraphicsContext at: offset - (cent - box topLeft // 2).
			spot displayOn: aGraphicsContext at: offset + (cent - box topLeft // 2)].
	(#(4 5 6) includes: model value)
		ifTrue: 
			[spot displayOn: aGraphicsContext at: offset - (cent - box topRight // 2).
			spot displayOn: aGraphicsContext at: offset + (cent - box topRight // 2)].
	6 = model value
		ifTrue: 
			[spot displayOn: aGraphicsContext at: offset - (cent - box leftCenter // 2).
			spot displayOn: aGraphicsContext at: offset + (cent - box leftCenter // 2)]! !

!DiceView methodsFor: 'bounds accessing'!

bounds: newBounds 
	"Re-compute my instance variables when my bounds changes"

	spotWidth := newBounds width // 5.
	spotHeight := newBounds height // 5.
	spot := (EllipticalArc boundingBox: (0 @ 0 extent: spotWidth @ spotHeight)) asFiller.
	^super bounds: newBounds! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DiceView class
	instanceVariableNames: ''!


!DiceView class methodsFor: 'example'!

example
	"DiceView example"
	| dice |
	dice := Dice new.
	self openOn: dice.
	dice inspect! !

!DiceView class methodsFor: 'instance creation'!

openOn: aDice 
	"Create an instance of the receiver in a ScheduledWindow"
	"NB: remember to modify Dice>>roll to include a #changed message"
	"DiceView openOn: Dice new"

	| window diceView wrapper | 
	window := ScheduledWindow 
					model: aDice
					label: 'Dice'
					minimumSize: 60 @ 60. 
	diceView := self model: aDice. 
	wrapper := BorderedWrapper on: diceView.
	wrapper insideColor: ColorValue white. 
	wrapper borderWidth: 2. 
	diceView controller: NoController new. 
	window component: wrapper. 
	window open! !

Model subclass: #Dice
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ex34.5'!
Dice comment:
'I am a Dice that provides a random number as my value. I inform my dependents when I change.

Instance Variables:
value	<Integer>	a number between 1 and 6'!


!Dice methodsFor: 'accessing'!

value 
	"Answer with the current value held by the receiver"

	^value! !

!Dice methodsFor: 'initialize-release'!

initialize
	"Initialize the variables of the receiver"

	self roll! !

!Dice methodsFor: 'modifying'!

roll
	"Set my value to be a random integer between 1 and 6"

	| rand | 
	rand := Random new.
	value := (rand next * 6 + 1) truncated.
	self changed! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Dice class
	instanceVariableNames: ''!


!Dice class methodsFor: 'instance creation'!

new
	"Answer with an initialized instance of the receiver"

	^super new initialize! !

